/* DroiDic 1.0
 * (c) Franois Pessaux May 2011.
 * This program is freeware.
 */

package francois.pessaux.droidic.android;

import java.io.IOException ;
import java.io.InputStream ;
import java.io.UnsupportedEncodingException ;

import francois.pessaux.androdict.android.R ;

import android.app.Activity;
import android.app.AlertDialog ;
import android.app.Dialog ;
import android.content.Context ;
import android.content.DialogInterface ;
import android.content.SharedPreferences ;
import android.content.res.Resources ;
import android.os.Bundle;
import android.view.KeyEvent ;
import android.view.Menu ;
import android.view.MenuInflater ;
import android.view.MenuItem ;
import android.view.View ;
import android.view.View.OnClickListener ;
import android.view.inputmethod.EditorInfo ;
import android.view.inputmethod.InputMethodManager ;
import android.webkit.WebView ;
import android.widget.Button ;
import android.widget.EditText ;
import android.widget.TextView ;
import android.widget.TextView.OnEditorActionListener ;


public class DroiDicMain extends Activity {
  private enum WorkingMode { WM_Spell, WM_Check, WM_Cheat } ;
  private WorkingMode currentWorkingMode = WorkingMode.WM_Spell ;
  private final CharSequence[] availableDicts = {
    "English", "Franais", "Italiano", "Norsk", "Portugus", "Espaol",
    "Svenska"
  } ;
  private int current_dic_index = 1 ;
  private final CharSequence[] caseSensitivityText = {
    "Case insensitive", "Case sentivite"
  };
  
  
  private final static int DIALOG_ID_ERR_DIC_RESOURCE = 0 ;
  private final static int DIALOG_ID_SELECT_DIC = 1 ;
  private final static int DIALOG_ID_ERR_ENCODING_CHARSET = 2 ;
  private final static int DIALOG_ID_ABOUT = 3 ;
  private final static int DIALOG_ID_SELECT_SENSITIVITY = 4 ;
  
  private TextView textViewLanguage ;
  private EditText editTextUserInput ;
  private Button searchButton ;
  private TextView textViewSuggestions ;
  
  private WebView webViewHelp ;
  
  
  private boolean initDictionaryResource (int dicId)
  {
    try {
      Resources resources = getResources () ;
      InputStream dictInputStream = resources.openRawResource (dicId) ;
      int fileSize ;
      fileSize = dictInputStream.available () ;
      byte rawData[] = new byte[fileSize] ;
      dictInputStream.read (rawData, 0, fileSize) ;
      dictInputStream.close () ;
      int rawInts[] = CLikeStringUtils.intArrayFromByteArray (rawData) ;
      return (DicManagement.loadDictionaryFromRawData (rawInts)) ;
      }
    catch (IOException e) {
      return (false) ;
      }
  }
  
  
  public void clearButtonOnClick (View view)
  {
    editTextUserInput.setText ("") ;
  }
  
  
  public void searchButtonOnClick (View view)
  {
    byte[] result ;
    String resultAsString ;
    
    /* First, hide the keyboard. */
    InputMethodManager imm =
        (InputMethodManager) getSystemService (Context.INPUT_METHOD_SERVICE) ;
    imm.hideSoftInputFromWindow (editTextUserInput.getWindowToken (), 0) ;

    try {
      byte[] inputStr =
        CLikeStringUtils.bytesCStringFromString
          (editTextUserInput.getText().toString()) ;
    
      switch (currentWorkingMode) {
      case WM_Spell:
        Suggest.suggest (inputStr) ;
        result = SuggBasics.getStringFromSuggestions () ;
        if (result != null) {
          resultAsString = new String (result, "ISO-8859-1") ;
          textViewSuggestions.setText (resultAsString) ;
        }
        else textViewSuggestions.setText (R.string.msgNoFoundSuggs) ;
        SuggBasics.releaseSuggestions () ;
        break ;
      
      case WM_Check:
        if (Exist.exists (inputStr))
          textViewSuggestions.setText (R.string.msgWordExists) ;
        else textViewSuggestions.setText (R.string.msgWordDoesntExist) ;
        break ;
      
      case WM_Cheat:
        Cheat.cheat (inputStr) ;
        result = SuggBasics.getStringFromSuggestions () ;
        if (result != null) {
          resultAsString = new String (result, "ISO-8859-1") ;
          textViewSuggestions.setText (resultAsString) ;
        }
        else textViewSuggestions.setText (R.string.msgNoFoundSuggs) ;
        SuggBasics.releaseSuggestions () ;
        break ;
      }
    }
    catch (UnsupportedEncodingException e) {
      showDialog (DIALOG_ID_ERR_ENCODING_CHARSET) ;
    }
  }
  
  
  private OnEditorActionListener userInputListener =
    new EditText.OnEditorActionListener () {
      public boolean onEditorAction (TextView v, int actionId, KeyEvent event)
      {
        if (actionId == EditorInfo.IME_ACTION_DONE) {
          searchButton.performClick () ;
          return (true) ;
        }
        return (false) ;
      }} ;
      
  
  private OnClickListener modeRadioButtonClickListener =
    new OnClickListener () {
      public void onClick (View v) {
        switch (v.getId()) {
        case R.id.radioButtonSuggest:
          currentWorkingMode = WorkingMode.WM_Spell ;
          break ;
        case R.id.radioButtonExists:
          currentWorkingMode = WorkingMode.WM_Check ;
          break ;
        case R.id.radioButtonCheat:
          currentWorkingMode = WorkingMode.WM_Cheat ;
          break ;
        }
        textViewSuggestions.setText ("") ;
    }} ;

  
  /* Strongly depends on the order the names of dictionaries are enumerated
     above ! */
  private int dicResIdFromDicIndex (int index)
  {
    int id = -1 ;
    
    switch (index) {
    case 0: id = R.raw.english ; break ;
    case 1: id = R.raw.french ; break ;
    case 2: id = R.raw.italian ; break ;
    case 3: id = R.raw.norwegian ; break ;
    case 4: id = R.raw.portuguese ; break ;
    case 5: id = R.raw.spanish ; break ;
    case 6: id = R.raw.swedish ; break ;
    }
    return (id) ;
  }
  
  
  
  @Override
  protected Dialog onCreateDialog (int id)
  {
    AlertDialog.Builder builder ;
    AlertDialog alert ;
    
    switch (id) {
    case DIALOG_ID_ERR_DIC_RESOURCE:
      builder = new AlertDialog.Builder (this) ;
      builder.setMessage (R.string.msgErrLoadingDic)
        .setCancelable (false)
        .setPositiveButton ("Quit", new DialogInterface.OnClickListener () {
          public void onClick (DialogInterface dialog, int id)
          {
            System.exit (-1) ;
          }}) ;
      alert = builder.create () ;
      alert.show () ;
      break ;
      
    case DIALOG_ID_SELECT_DIC:
      builder = new AlertDialog.Builder (this) ;
      builder.setTitle (R.string.titleSelectDict) ;
      builder.setItems (availableDicts, new DialogInterface.OnClickListener () {
          public void onClick (DialogInterface dialog, int item)
          {
            int dicId = dicResIdFromDicIndex (item) ;
            if (dicId != -1) {
              current_dic_index = item ;
              DicManagement.unloadDictionary () ;
              initDictionaryResource (dicId) ;
              textViewLanguage.setText (availableDicts[item]) ;
            }
          }}) ;
      alert = builder.create () ;
      alert.show () ;
      break ;
      
    case DIALOG_ID_ERR_ENCODING_CHARSET:
      builder = new AlertDialog.Builder (this) ;
      builder.setMessage (R.string.msgInvalidCharSet)
        .setCancelable (true)
        .setPositiveButton ("Ok", new DialogInterface.OnClickListener () {
          public void onClick (DialogInterface dialog, int id) { }}) ;
      alert = builder.create () ;
      alert.show () ;
      break ;
      
    case DIALOG_ID_ABOUT:
      builder = new AlertDialog.Builder (this) ;
      builder.setMessage ("DroiDic 1.0 (c) Franois Pessaux 2011.")
        .setCancelable (true)
        .setPositiveButton ("Ok", new DialogInterface.OnClickListener () {
          public void onClick (DialogInterface dialog, int id) { }}) ;
      alert = builder.create () ;
      alert.show () ;
      break ;
    
    case DIALOG_ID_SELECT_SENSITIVITY:
      builder = new AlertDialog.Builder (this) ;
      builder.setTitle (R.string.titleSelectSensitivity) ;
      builder.setItems (caseSensitivityText, new DialogInterface.OnClickListener () {
        public void onClick (DialogInterface dialog, int item)
        { 
          switch (item) {
          case 0: Global.case_sensitive = false ; break ;
          case 1: Global.case_sensitive = true ; break ;
          }
        }}) ;
      alert = builder.create () ;
      alert.show () ;
      break ;
      }
      
    return (super.onCreateDialog (id)) ;
  }  

  
  private void installMainView ()
  {
    setContentView (R.layout.main) ;
    textViewLanguage = (TextView) findViewById (R.id.textViewLanguage) ;
    editTextUserInput = (EditText) findViewById (R.id.editTextUserInput) ;
    editTextUserInput.setOnEditorActionListener (userInputListener) ;
    searchButton = (Button) findViewById (R.id.buttonSearch) ;
    textViewSuggestions = (TextView) findViewById (R.id.textViewSuggestions) ;
    findViewById (R.id.radioButtonCheat).setOnClickListener (modeRadioButtonClickListener) ;
    findViewById (R.id.radioButtonExists).setOnClickListener (modeRadioButtonClickListener) ;
    findViewById (R.id.radioButtonSuggest).setOnClickListener (modeRadioButtonClickListener) ;
    currentWorkingMode = WorkingMode.WM_Spell ;
  }
  
  
  
  /** Called when the activity is first created. */
  @Override
  public void onCreate (Bundle savedInstanceState)
  {
    super.onCreate (savedInstanceState)  ;
    installMainView () ;
    /* Restore last preferences. */
    SharedPreferences settings = getPreferences (MODE_PRIVATE) ;
    Global.case_sensitive = settings.getBoolean ("caseSensitive", false) ;
    current_dic_index = settings.getInt ("dicIndex", 1) ;
    int dicResId = dicResIdFromDicIndex (current_dic_index) ;
    if (! initDictionaryResource (dicResId))
      showDialog (DIALOG_ID_ERR_DIC_RESOURCE) ;
    textViewLanguage.setText (availableDicts[current_dic_index]) ;
  }
  
  
  private void savePrefs ()
  {
    /* We need an Editor object to make preference changes. */
    SharedPreferences settings = getPreferences (MODE_PRIVATE) ;
    SharedPreferences.Editor editor = settings.edit () ;
    editor.putBoolean ("caseSensitive", Global.case_sensitive) ;
    editor.putInt ("dicIndex", current_dic_index) ;
    
    /* Commit the edits. */
    editor.commit () ;
  }
  
  
  @Override
  protected void onStop ()
  {
    savePrefs () ;
    super.onStop () ;
  }
  
  
  @Override
  public void finish ()
  {
    savePrefs () ;
    DicManagement.unloadDictionary () ;
    System.exit (0) ;
  }
  
  
  @Override
  public boolean onCreateOptionsMenu (Menu menu)
  {
    MenuInflater inflater = getMenuInflater () ;
    inflater.inflate (R.menu.menu, menu) ;
    return (true) ;
  }
  
  
  @Override
  public boolean onOptionsItemSelected (MenuItem item)
  {
    switch (item.getItemId ()) {
    case R.id.item_dictionaries:
      showDialog (DIALOG_ID_SELECT_DIC) ;
      return (true) ;
      // break ;
      
    case R.id.itemcase_sens:
      showDialog (DIALOG_ID_SELECT_SENSITIVITY) ;
      return (true) ;
      // break ;
      
    case R.id.item_help:
      setContentView (R.layout.help) ;
      webViewHelp = (WebView) findViewById (R.id.webViewHelp) ;
      // webViewHelp.loadUrl ("file:///android_asset/droidic_help_en.html") ;
      webViewHelp.loadUrl (getString(R.string.fileNameHelp)) ;
      return (true) ;
      // break ;

    case R.id.item_about:
      showDialog (DIALOG_ID_ABOUT) ;
      return (true) ;
      // break ;

    case R.id.item_exit:
      this.finish () ;
      return (true) ;
      // break ;

    default:
      return (super.onOptionsItemSelected (item)) ;
      // break ;
    }
  }
  
  
  public void dismissHelpButtonOnClick (View view)
  {
    installMainView () ;
  }
  
  
  public void backHelpButtonOnClick (View view)
  {
    if ((webViewHelp != null) &&  (webViewHelp.canGoBack ()))
      webViewHelp.goBack() ;
  }
  
}
